###############################################################################
# Copyright (c) Lawrence Livermore National Security, LLC and other Ascent
# Project developers. See top-level LICENSE AND COPYRIGHT files for dates and
# other details. No copyright assignment is required to contribute to Ascent.
###############################################################################

###############################################################################
# Note:
# This file is named AscentConfig.cmake because once upon a time
# when it was named ascent-config.cmake, we found that CMake's exported
# targets script includes all "ascent-*.cmake" files. This logic would
# cause this script to be included more than once, seeding instability
# that caused great harm to the kingdom.
###############################################################################

cmake_minimum_required(VERSION 3.0 FATAL_ERROR)


####### Expanded from @PACKAGE_INIT@ by configure_package_config_file() #######
####### Any changes to this file will be overwritten by the next CMake run ####
####### The input file was AscentConfig.cmake.in                            ########

get_filename_component(PACKAGE_PREFIX_DIR "${CMAKE_CURRENT_LIST_DIR}/../../../" ABSOLUTE)

macro(set_and_check _var _file)
  set(${_var} "${_file}")
  if(NOT EXISTS "${_file}")
    message(FATAL_ERROR "File or directory ${_file} referenced by variable ${_var} does not exist !")
  endif()
endmacro()

macro(check_required_components _NAME)
  foreach(comp ${${_NAME}_FIND_COMPONENTS})
    if(NOT ${_NAME}_${comp}_FOUND)
      if(${_NAME}_FIND_REQUIRED_${comp})
        set(${_NAME}_FOUND FALSE)
      endif()
    endif()
  endforeach()
endmacro()

####################################################################################


if(NOT ASCENT_FOUND)

    set(ASCENT_VERSION "0.9.2")
    set(ASCENT_INSTALL_PREFIX "/usr/local")
    set(ASCENT_CONDUIT_DIR  "/usr/local")

    # advertise if cuda is enabled
    set(ASCENT_CUDA_ENABLED "OFF")

    # advertise if hip is enabled
    set(ASCENT_HIP_ENABLED "OFF")
    set(ASCENT_ROCM_PATH "")
    
    if(ASCENT_CUDA_ENABLED OR ASCENT_HIP_ENABLED)
        set(ASCENT_DEVICE_ENABLED TRUE)
    endif()

    # advertise if openmp is enabled
    set(ASCENT_OPENMP_ENABLED "OFF")

    # advertise if fortran support is enabled
    set(ASCENT_FORTRAN_ENABLED ON)

    # advertise if caliper support is enabled
    set(ASCENT_CALIPER_ENABLED )

    set(ASCENT_ADIAK_DIR "")
    set(ASCENT_CALIPER_DIR "")
    
    # advertise if camp support is enabled
    set(ASCENT_CAMP_ENABLED )
    set(ASCENT_CAMP_DIR )

    # advertise if umpire support is enabled
    set(ASCENT_UMPIRE_ENABLED )
    set(ASCENT_UMPIRE_DIR )

    # advertise if raja support is enabled
    set(ASCENT_RAJA_ENABLED )
    set(ASCENT_RAJA_DIR )

    # advertise if apcomp is enabled
    set(ASCENT_APCOMP_ENABLED OFF)

    # advertise if vtk-h support is enabled
    set(ASCENT_VTKH_ENABLED OFF)

    # vtk-m dir
    set(ASCENT_VTKM_DIR "")

    # advertise if mfem support is enabled
    set(ASCENT_MFEM_ENABLED )
    set(ASCENT_MFEM_DIR "")
    set(ASCENT_MFEM_MPI_ENABLED )

    # advertise if devil ray support is enabled
    set(ASCENT_DRAY_ENABLED OFF)

    # advertise if occa support is enabled
    set(ASCENT_OCCA_ENABLED )
    set(ASCENT_OCCA_DIR "")

    # advertise if umpire support is enabled
    set(ASCENT_UMPIRE_ENABLED )
    set(ASCENT_UMPIRE_DIR "")

    # advertise if camp support is enabled
    set(ASCENT_CAMP_ENABLED )
    set(ASCENT_CAMP_DIR "")

    ##################################################
    # advertise if babelflow support is enabled
    set(ASCENT_BABELFLOW_ENABLED )
    set(ASCENT_BABELFLOW_DIR "")
    set(ASCENT_PMT_DIR "")
    set(ASCENT_STREAMSTAT_DIR "")
    set(ASCENT_TOPOFILEPARSER_DIR "")

    ##################################################
    # advertise if fides and adios2 support is enabled
    set(ASCENT_FIDES_ENABLED )
    set(ASCENT_FIDES_DIR "")

    set(ASCENT_ADIOS2_ENABLED )
    set(ASCENT_ADIOS2_DIR "")

    # advertise if genten support is enabled
    set(ASCENT_GENTEN_ENABLED )
    set(ASCENT_GENTEN_DIR "")

    # advertise if Kokkos support is enabled
    set(ASCENT_KOKKOS_ENABLED )
    set(ASCENT_KOKKOS_DIR "")

    # advertise if python support is enabled
    set(ASCENT_PYTHON_ENABLED )
    set(ASCENT_PYTHON_EXECUTABLE "")

    set(ASCENT_SERIAL_ENABLED ON)
    set(ASCENT_MPI_ENABLED ON)

    # pull in vars with details about configured paths
    get_filename_component(ASCENT_CMAKE_CONFIG_DIR "${CMAKE_CURRENT_LIST_FILE}" PATH)

    # setup dependent pkgs (conduit + vtk-h)
    include(${ASCENT_CMAKE_CONFIG_DIR}/ascent_setup_deps.cmake)

    # include the main exported targets
    include("${ASCENT_CMAKE_CONFIG_DIR}/ascent.cmake")

    # finish setup
    include("${ASCENT_CMAKE_CONFIG_DIR}/ascent_setup_targets.cmake")

    set(ASCENT_FOUND TRUE)

endif()
